package dk.hansen;

import org.apache.log4j.Logger;

import dk.hansen.dao.DAOException;
import dk.hansen.dao.DAOFactoryIF;
import dk.hansen.dao.DVDDAOIF;
import dk.hansen.dao.DirectorDAOIF;
import dk.hansen.dao.JdbcDAOFactory;
import dk.hansen.dao.bean.DVD;
import dk.hansen.dao.bean.Director;
import dk.hansen.dto.ConvertKey;
import dk.hansen.dto.DVDWithDirector_DTO;
import dk.hansen.dto.DVD_DTO;
import dk.hansen.dto.Director_DTO;
import dk.hansen.transaction.TransactionIF;

public class TransactionHandler {

  private static Logger logger = Logger.getLogger(TransactionHandler.class.getName());

  private TransactionIF trans;

  private DVDDAOIF dvdDao;

  private DirectorDAOIF directorDao;

  public TransactionHandler() throws DAOException {
    //  DAOFactoryIF factory = new SimpleDAOFactory();
    DAOFactoryIF factory = new JdbcDAOFactory();
    trans = factory.createTransaction();
    dvdDao = factory.createDVDDAO();
    directorDao = factory.createDirectorDAO();
  }

  public DVD_DTO findDVD(String dvdId) throws DAOException {
    DVD_DTO dto = null;

    DAOFactoryIF factory = new JdbcDAOFactory();
    DVDDAOIF dvdDao = factory.createDVDDAO();
    dvdDao.setTransaction(trans);
    DVD dvd;
    try {
      dvd = dvdDao.findDVD(dvdId);
    } catch (DAOException e) {
      String msg = "findDVD failed for id=" + dvdId;
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    if (dvd != null) {
      dto = new DVD_DTO();
      dto.setKey(ConvertKey.toDTO(dvd.getKey()));
      dto.setTitle(dvd.getTitle());
      dto.setDirectorKey(ConvertKey.toDTO(dvd.getDirector()));
    }
    return dto;
  }

  public DVDWithDirector_DTO findDVDWithDirector(String dvdId) throws DAOException {
    DVDWithDirector_DTO dvdDTO = null;

    DAOFactoryIF factory = new JdbcDAOFactory();
    DVDDAOIF dvdDao = factory.createDVDDAO();
    DirectorDAOIF directorDao = factory.createDirectorDAO();
    dvdDao.setTransaction(trans);
    DVD dvd;
    try {
      dvd = dvdDao.findDVD(dvdId);
    } catch (DAOException e) {
      String msg = "getDVD failed for id=" + dvdId;
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    if (dvd != null) {
      directorDao.setTransaction(trans);
      Director director;
      try {
        director = directorDao.getDirector(dvd.getDirector().getId());
      } catch (DAOException e) {
        String msg = "getDVD failed for directorid=" + dvd.getDirector().getId();
        logger.error(msg);
        throw new DAOException(msg, e);
      }

      Director_DTO directorDTO = new Director_DTO();
      directorDTO.setKey(ConvertKey.toDTO(director.getKey()));
      directorDTO.setName(director.getName());

      dvdDTO = new DVDWithDirector_DTO();
      dvdDTO.setKey(ConvertKey.toDTO(dvd.getKey()));
      dvdDTO.setTitle(dvd.getTitle());
      dvdDTO.setDirector(directorDTO);
    }
    return dvdDTO;
  }

  public void create(DVD_DTO dvd) throws DAOException {
    try {
      trans.begin();
    } catch (DAOException e) {
      String msg = "'begin transaction' failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    try {
      dvdDao.setTransaction(trans);
      dvdDao.createDVD(dvd.getKey().getId(), dvd.getTitle(), dvd.getDirectorKey().getId());
    } catch (DAOException e) {
      try {
        trans.rollback();
        logger.info("'rollback' successful.");
      } catch (DAOException e1) {
        logger.error("'rollback transaction' failed", e1);
      }
      String msg = "create failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    try {
      trans.end();
    } catch (DAOException e) {
      String msg = "'end transaction' failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }
  }

  public void delete(DVD_DTO dvd) throws DAOException {
    try {
      trans.begin();
    } catch (DAOException e) {
      String msg = "'begin transaction' failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    try {
      dvdDao.setTransaction(trans);
      dvdDao.deleteDVD(dvd.getKey().getId());
    } catch (DAOException e) {
      try {
        trans.rollback();
        logger.info("'rollback' successful.");
      } catch (DAOException e1) {
        logger.error("'rollback transaction' failed", e1);
      }
      String msg = "delete failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    try {
      trans.end();
    } catch (DAOException e) {
      String msg = "'end transaction' failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }
  }

  /**
   * Rename a DVD by id by appending a text to its id
   * 
   * @param id
   * @param append
   * @throws DAOException
   */
  public void rename(String id, String append) throws DAOException {
    dvdDao.setTransaction(trans);
    DVD dvd = dvdDao.findDVD(id);
    if (dvd == null)
      throw new DAOException("DVD " + id + " not found");

    try {
      trans.begin();
    } catch (DAOException e) {
      String msg = "'begin transaction' failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    try {
      dvdDao.deleteDVD(id);
      dvdDao.createDVD(id + append, dvd.getTitle(), dvd.getDirector().getId());
    } catch (DAOException e) {
      try {
        trans.rollback();
        logger.info("'rollback' successful.");
      } catch (DAOException e1) {
        logger.error("'rollback transaction' failed", e1);
      }
      String msg = "rename failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }

    try {
      trans.end();
    } catch (DAOException e) {
      String msg = "'end transaction' failed.";
      logger.error(msg);
      throw new DAOException(msg, e);
    }
  }

  public static void main(String[] args) throws DAOException {
    TransactionHandler handler = new TransactionHandler();
    //    handler.rename("ID1", "-2005");
    //    DVD_DTO dto = handler.getDVD("ID2");
    //    System.out.println("dvd directorKey=" + dto.getDirectorKey().getId());

    DVD_DTO dto1 = handler.findDVD("ID2");
    System.out.println("dvd directorKey=" + dto1.getDirectorKey().getId());

    DVDWithDirector_DTO dto2 = handler.findDVDWithDirector("ID2");
    if (dto2 != null) {
      System.out.println("dvd directorKey=" + dto2.getDirector().getKey().getId());
      System.out.println("dvd directorName=" + dto2.getDirector().getName());
    }

    DVDWithDirector_DTO dto3 = handler.findDVDWithDirector("ID2x");
    if (dto3 != null) {
      System.out.println("dvd directorKey=" + dto3.getDirector().getKey().getId());
      System.out.println("dvd directorName=" + dto3.getDirector().getName());
    }
  }
}