package dk.hansen.unittest;

import java.util.Collection;
import java.util.Iterator;

import dk.hansen.dao.DAOException;
import dk.hansen.dao.DAOFactoryIF;
import dk.hansen.dao.DVDDAOIF;
import dk.hansen.dao.bean.DVD;
import dk.hansen.dao.bean.Key;
import dk.hansen.transaction.TransactionIF;

import junit.framework.TestCase;

public class UnitTest2 extends TestCase {

  private DVDDAOIF dao;
  private TransactionIF trans;
  
  protected void setUp() throws Exception {
    // Create a factory class
    String managerName = "dk.hansen.dao.JdbcDAOFactory";
    //String managerName = "dk.hansen.dao.SimpleDAOFactory";
    DAOFactoryIF factory = (DAOFactoryIF) Class.forName(managerName).newInstance();

    // Create a transaction and a DAO
    trans = factory.createTransaction();
    dao = factory.createDVDDAO();
    dao.setTransaction(trans);
    
    
    String id1 = "ID1";
    String id2 = "ID2";
    String id3 = "ID3";
    String id4 = "ID4";

    try {
      dao.deleteDVD(id1);
    } catch (Exception e) {}
    try {
      dao.deleteDVD(id2);
    } catch (Exception e) {}
    try {
      dao.deleteDVD(id3);
    } catch (Exception e) {}
    try {
      dao.deleteDVD(id4);
    } catch (Exception e) {}

    dao.createDVD(id1, "Lord of the Rings (1)", "1");
    dao.createDVD(id2, "The Matrix", "1");

  }

  public void testfind() throws DAOException {
    String id;
    DVD dvd;

    id = "ID1";
    dvd = dao.findDVD(id);
    assertNotNull("ID1 must be found", dvd);
    assertEquals("Test for ID1", id, dvd.getKey().getId());
    System.out.println("*** ID=" + dvd.getDirector().getId());
    //System.out.println("*** Dirname=" + dvd.getDirectorKey());


    id = "ID2";
    dvd = dao.findDVD(id);
    assertNotNull("ID2 must be found", dvd);
    assertEquals("Test for ID2", id, dvd.getKey().getId());

    assertEquals("Test for ID2 - title", "The Matrix", dvd.getTitle());

    id = "ID3";
    dvd = dao.findDVD(id);
    assertNull("Test for ID3", dvd);
  }

  public void testupdate() throws DAOException {
    String id;
    DVD dvd;

    id = "ID2";
    dvd = dao.findDVD(id);
    assertNotNull("ID2 must be found", dvd);
    assertEquals("Test for ID2", id, dvd.getKey().getId());

    String newTitle = "Troy";
    dao.updateDVD(id, newTitle);
    dvd = dao.findDVD(id);
    assertNotNull("ID2 must be found", dvd);
    assertEquals("Test for new title", newTitle, dvd.getTitle());

  }

  public void testcreate() throws DAOException {
    String id;
    String title;
    String dirId;
    DVD dvd;

    id = "ID3";
    title = "Troy";
    dirId = "1";

    dao.createDVD(id, title, dirId);
    dvd = dao.findDVD(id);
    assertNotNull("ID3 must be found", dvd);
    assertEquals("Test for ID3", title, dvd.getTitle());

  }

  public void testdelete() throws DAOException {
    String id;
    DVD dvd;

    id = "ID2";
    dvd = dao.findDVD(id);
    assertNotNull("ID2 must be found", dvd);
    assertEquals("Test for ID2", id, dvd.getKey().getId());

    dao.deleteDVD(id);
    dvd = dao.findDVD(id);
    assertNull("Test for deleted ID2", dvd);

  }

  public void testall() throws DAOException {
    String id;
    DVD dvd;
    Collection c;

    c = dao.findAll();
    assertTrue("Number of DVDs", c.size() >= 2);
  }

  public void testallKeys() throws DAOException {
    String id;
    DVD dvd;
    Collection c;

    c = dao.findAllDVDKeys();
    assertTrue("Number of DVDs", c.size() >= 2);
    
    Iterator it = c.iterator();
    while (it.hasNext()) {
      Key key = (Key)it.next();
      assertTrue("DVD keys", !key.getId().equals(""));
    }
  }

  public void testfindtitle() throws DAOException {
    String id;
    DVD dvd;
    Collection c;

    c = dao.findAllDVDWithTitle("xyz");
    assertEquals("search for xyz", 0, c.size());

    c = dao.findAllDVDWithTitle("The Matrix");
    assertEquals("search for The Matrix", 1, c.size());
    assertEquals("title Matrix", "The Matrix", ((DVD) c.iterator().next())
        .getTitle());

    c = dao.findAllDVDWithTitle("The Matrix");
    assertEquals("search for Matrix", 1, c.size());

    c = dao.findAllDVDWithTitle("The");
    assertTrue("search for word", c.size() >= 2);

  }

  /**
   * Simple, OK transaction 
   */
  public void testTransaction1() throws DAOException {
    trans.begin();
    testcreate();
    trans.end();
  }
  
  /**
   * Double insert must throw an exception
   * @throws DAOException
   */
  public void testTransaction2() throws DAOException {
    trans.begin();
    testcreate();
    try {
      testcreate();
      fail("Double create must throw an exception");
    } catch (DAOException e) {
      trans.rollback();
    }
  }
  
  /**
   * Check that rollback works
   * @throws DAOException
   */
  public void testTransaction3() throws DAOException {
    trans.begin();
    testcreate();
    try {
      testcreate();
      fail("Double create must throw an exception");
    } catch (DAOException e) {
      trans.rollback();
    }
    // Check that the inserted record has been rolled back
    testfind();
  }
  
  public void testTransaction4() throws DAOException {
    String id = "ID4";
    String title = "Tarzan";
    String dirId = "1";
    dao.createDVD(id, title, dirId);
    
    
    trans.begin();

    id = "ID3";
    title = "Troy";
    dao.createDVD(id, title, dirId);
    
    DVD dvd = dao.findDVD(id);
    assertNotNull("Test for ID3", dvd);
    
    try {
      dao.createDVD(id, title, dirId);
      fail("Double create must throw an exception");
    } catch (DAOException e) {
      trans.rollback();
    }
    
    // Check that the inserted record has been rolled back - and nothing else...
    dvd = dao.findDVD(id);
    assertNull("Test for ID3", dvd);
    
    dvd = dao.findDVD("ID4");
    assertNotNull("Test for ID4", dvd);
    
    testTransaction1();
  }
  
  
  public static void main(String[] args) {
    junit.textui.TestRunner.run(UnitTest2.class);
  }

}